"""streamparse.ext.invoke

invoke tasks that are imported into a project's task.py.

Should be used like this::

    from invoke import task, run
    from streamparse.ext.invoke import *

    # your other tasks
"""
from __future__ import absolute_import, print_function
import os
import shutil
from tempfile import NamedTemporaryFile
import re

from invoke import run, task

from ..contextmanagers import ssh_tunnel
from .util import (get_env_config, get_topology_definition,
                   get_nimbus_for_env_config, get_config)
from .fabric import activate_env, create_or_update_virtualenvs, tail_logs



__all__ = ["list_topologies",  "kill_topology", "run_local_topology",
           "submit_topology", "tail_topology"]

# TODO: remove boilerplate get_env_config, get_nimbus_for_env_config...
# from all these with something like
# @task("setup")

@task
def prepare_topology():
    """Prepare a topology for running locally or deployment to a remote
    cluster.
    """
    if os.path.isdir("_resources/resources"):
        shutil.rmtree("_resources/resources")
    shutil.copytree("src", "_resources/resources")


@task
def list_topologies(env_name="prod"):
    env_name, env_config = get_env_config(env_name)
    host, port = get_nimbus_for_env_config(env_config)

    with ssh_tunnel(env_config["user"], host, 6627, port):
        cmd = ["lein",
               "run -m streamparse.commands.list/-main"]
        run(" ".join(cmd))


@task
def kill_topology(topology_name=None, env_name="prod"):
    topology_name, topology_file = get_topology_definition(topology_name)
    env_name, env_config = get_env_config(env_name)
    host, port = get_nimbus_for_env_config(env_config)

    with ssh_tunnel(env_config["user"], host, 6627, port):
        cmd = ["lein",
               "run -m streamparse.commands.kill_topology/-main",
               topology_name]
        run(" ".join(cmd))


@task
def jar_for_deploy():
    print("Cleaning from prior builds...")
    res = run("lein clean", hide="stdout")
    if not res.ok:
        raise Exception("Unable to run 'lein clean'!\nSTDOUT:\n{}"
                        "\nSTDERR:\n{}".format(res.stdout, res.stderr))
    print("Creating topology JAR...")
    res = run("lein jar", hide="stdout")
    if not res.ok:
        raise Exception("Unable to run 'lein jar'!\nSTDOUT:\n{}"
                        "\nSTDERR:\n{}".format(res.stdout, res.stderr))
    # XXX: This will fail if more than one JAR is built
    lines = [l.strip().lstrip("Created ") for l in res.stdout.split()
             if l.endswith(".jar")]
    return lines[0]


@task(pre=["prepare_topology"])
def run_local_topology(name=None, time=5, debug=False):
    """Run a topology locally using Storm's LocalCluster class."""
    prepare_topology()

    name, topology_file = get_topology_definition(name)
    print("Running {} topology...".format(name))
    cmd = ["lein run -m streamparse.commands.run/-main", topology_file,
           "-t", str(time)]
    if debug:
        cmd.append("--debug")
    run(" ".join(cmd))


@task(pre=["prepare_topology"])
def submit_topology(name=None, env_name="prod", debug=False):
    """Submit a topology to a remote Storm cluster."""
    prepare_topology()

    config = get_config()
    name, topology_file = get_topology_definition(name)
    env_name, env_config = get_env_config(env_name)
    host, port = get_nimbus_for_env_config(env_config)

    activate_env(env_name)
    create_or_update_virtualenvs(name,
                                 "{}/{}.txt".format(config["virtualenv_specs"],
                                                    name))

    # TODO: Super hacky way to replace "python" with our venv executable, need
    # to fix this
    with open(topology_file, "r") as fp:
        contents = fp.read()
    contents = re.sub(r'"python"',
                     '"{}/{}/bin/python"'
                      .format(env_config["virtualenv_path"], name),
                      contents)
    tmpfile = NamedTemporaryFile(dir=config["topology_specs"])
    tmpfile.write(contents)
    tmpfile.flush()
    print("Created modified topology definition file {}.".format(tmpfile.name))

    # replaced with /path/to/venv/bin/python instead

    # Prepare a JAR that doesn't have Storm dependencies packaged
    topology_jar = jar_for_deploy()

    print('Deploying "{}" topology...'.format(name))
    with ssh_tunnel(env_config["user"], host, 6627, port):
        print("ssh tunnel to Nimbus {}:{} established."
              .format(host, port))
        jvm_opts = [
            "-Dstorm.jar={}".format(topology_jar),
            "-Dstorm.options=",
            "-Dstorm.conf.file=",
        ]
        os.environ["JVM_OPTS"] = " ".join(jvm_opts)
        cmd = ["lein",
               "run -m streamparse.commands.submit_topology/-main",
               tmpfile.name,
               "--debug"]
        run(" ".join(cmd))

    tmpfile.close()

@task
def tail_topology(env_name="prod"):
    activate_env(env_name)
    tail_logs()



"""Base Spout classes."""
from __future__ import print_function

from ipc import read_handshake, read_command, send_message, json, _stdout
from base import Component


class Spout(Component):
    """Base class for all streamparse spouts.  For more information on spouts,
    consult Storm's `Concepts documentation <http://storm.incubator.apache.org/documentation/Concepts.html>`_.
    """

    def initialize(self, storm_conf, context):
        """Called immediately after the initial handshake with Storm and before
        the main run loop. A good place to initialize connections to data
        sources.

        :param storm_conf: a ``dict`` containing the Storm configuration for
        this spout.  This is the configuration provided to the topology, merged
        in with cluster configuration on the worker node.
        :param context: a ``dict`` containing information about the component's
        place within the topology such as: task IDs, inputs, outputs etc."""
        pass

    def ack(self, tup_id):
        """Called when a bolt acknowledges a tuple in the topology.

        :param tup_id: a ``str`` representing the tuple that was acknowledged.
        """
        pass

    def fail(self, tup_id):
        """Called when a tuple fails in the topology, a Spout can choose to
        emit the tuple again or ignore the fail.  Default is to ignore.

        :param tup_id: a ``str`` representing the tuple that was failed.
        """
        pass

    def next_tuple(self):
        raise NotImplementedError()

    def emit(self, tup, tup_id=None, stream=None, direct_task=None):
        """Emit a spout tuple message.

        :param tup: a ``list`` representing the tuple to send to Storm.  Should
        contain only JSON-serializable data.
        :param tup_id: ``str`` or ``int`` which is the id for the tuple. Leave
        this blank for an unreliable emit.
        :param stream: ``str`` ID of the stream this tuple should be emitted
        to.  Leave empty to emit to the default stream.
        :param direct_task: ``int`` indicating the task to send the tuple to if
        performing a direct emit.
        """
        if not isinstance(tup, list):
            raise TypeError('All tuples must be lists, received {!r} instead'
                            .format(type(tup)))

        msg = {'command': 'emit', 'tuple': tup}
        if tup_id is not None:
            msg['id'] = tup_id
        if stream is not None:
            msg['stream'] = stream
        if direct_task is not None:
            msg['task'] = direct_task

        send_message(msg)

    def emit_many(self, tuples, tup_id=None, stream=None, direct_task=None):
        """A more efficient way to send many tuples, dumps out all tuples to
        STDOUT instead of writing one at a time.

        :param tuples: a two-dimensional ``list`` representing the tuples to
        send to Storm.  Tuples should contain only JSON-serializable data.
        :param tup_id: ``str`` or ``int`` which is the id for the tuple.  Leave
        this blank for unreliable emits.
        :param stream: ``str`` ID of the stream these tuples should be emitted
        to.  Leave empty to emit to the default stream.
        :param direct_task: ``int`` indicating the task to send the tuple to if
        performing a direct emit.
        """
        msg = {
            'command': 'emit',
        }
        if tup_id is not None:
            msg['id'] = tup_id
        if stream is not None:
            msg['stream'] = stream
        if direct_task is not None:
            msg['task'] = direct_task

        lines = []
        for tup in tuples:
            msg['tuple'] = tup
            lines.append(json.dumps(msg))
            lines.append('end')
        print('\n'.join(lines), file=_stdout)

    def run(self):
        """Main run loop for all spouts. Performs initial handshake with Storm
        and reads tuples handing them off to subclasses.  Any exceptions are
        caught and logged back to Storm prior to the Python process exits.
        Subclasses should not override this method.
        """
        storm_conf, context = read_handshake()
        try:
            self.initialize(storm_conf, context)
            while True:
                cmd = read_command()
                if cmd['command'] == 'next':
                    self.next_tuple()
                if cmd['command'] == 'ack':
                    self.ack(cmd['id'])
                if cmd['command'] == 'fail':
                    self.fail(cmd['id'])
                send_message({'command': 'sync'})
        except Exception as e:
            self.raise_exception(e)



"""Base Bolt classes."""
from __future__ import print_function

from base import Component
from util import read_handshake, read_tuple, send_message, json, _stdout


_ANCHOR_TUPLE = None


class Bolt(Component):
    """The base class for all streamparse bolts. For more information on bolts,
    consult Storm's `Concepts documentation <http://storm.incubator.apache.org/documentation/Concepts.html>`_.
    """

    def initialize(self, storm_conf, context):
        """Called immediately after the initial handshake with Storm and before
        the main run loop. A good place to initialize connections to data
        sources.

        :param storm_conf: a ``dict`` containing the Storm configuration for
        this spout.  This is the configuration provided to the topology, merged
        in with cluster configuration on the worker node.
        :param context: a ``dict`` containing information about the component's
        place within the topology such as: task IDs, inputs, outputs etc.
        """
        pass

    def process(self, tup):
        """Process a single tuple :class:`base.Tuple` of input, should be
        overridden by subclasses.  :class:`base.Tuple` objects contain metadata
        about which component, stream and task it came from.  The actual values
        of the tuple can be accessed by calling `tup.values`.

        :param tup: a ``Tuple`` object.
        """
        raise NotImplementedError()

    def emit(self, tup, stream=None, anchors=[], direct_task=None):
        """Emit a new tuple to a stream.

        :param tup: a ``list`` representing the tuple to send to Storm, should
        contain only JSON-serializable data.
        :param stream: a ``str`` indicating the ID of the stream to emit this
        tuple to. Specify ``None`` to emit to default stream.
        :param anchors: a ``list`` of IDs of the tuples tuple should be
        anchored to.
        :param direct_task: an ``int`` which indicates the task to send the
        tuple to.
        """
        if not isinstance(tup, list):
            raise TypeError('All tuples must be lists, received {!r} instead'
                            .format(type(tup)))
        if _ANCHOR_TUPLE is not None:
            anchors = [_ANCHOR_TUPLE]
        msg = {'command': 'emit', 'tuple': tup}
        if stream is not None:
            msg['stream'] = stream
        msg['anchors'] = [x.id for x in anchors]
        if direct_task is not None:
            msg['task'] = direct_task

        send_message(msg)

    def emit_many(self, tuples, stream=None, anchors=[], direct_task=None):
        """A more efficient way to send many tuples, dumps out all tuples to
        STDOUT instead of writing one at a time.

        :param tuples: an iterable of ``list``s representing the tuples to send
        to Storm. All tuples should contain only JSON-serializable data.
        :param stream: a ``str`` indicating the ID of the steram to emit these
        tuples to. Specify ``None`` to emit to default stream.
        :param anchors: a `` list`` of IDs the tuples these tuples should be
        anchored to.
        :param direct_task: an ``int`` which indicates the task to send the
        tuple to.
        """
        if _ANCHOR_TUPLE is not None:
            anchors = [_ANCHOR_TUPLE]
        msg = {
            'command': 'emit',
            'anchors': [a.id for a in anchors],
        }
        if stream is not None:
            msg['stream'] = stream
        if direct_task is not None:
            msg['task'] = direct_task

        lines = []
        for tup in tuples:
            msg['tuple'] = tup
            lines.append(json.dumps(msg))
            lines.append('end')
        print('\n'.join(lines), file=_stdout)

    def ack(self, tup):
        """Indicate that processing of a tuple has succeeded.

        :param tup: a :class:`base.Tuple` object.
        """
        send_message({'command': 'ack', 'id': tup.id})

    def fail(self, tup):
        """Indicate that processing of a tuple has failed.

        :param tup: a :class:`base.Tuple` object.
        """
        send_message({'command': 'fail', 'id': tup.id})

    def run(self):
        """Main run loop for all bolts. Performs initial handshake with Storm
        and reads tuples handing them off to subclasses.  Any exceptions are
        caught and logged back to Storm prior to the Python process exits.
        Subclasses should not override this method.
        """
        storm_conf, context = read_handshake()
        tup = None
        try:
            self.initialize(storm_conf, context)
            while True:
                tup = read_tuple()
                self.process(tup)
        except Exception as e:
            self.raise_exception(e, tup)


class BasicBolt(Bolt):
    """A bolt implementation that automatically acknowledges all tuples after
    `self.process(tup)` completes."""

    def run(self):
        global _ANCHOR_TUPLE
        storm_conf, context = read_handshake()
        tup = None
        try:
            self.initialize(storm_conf, context)
            while True:
                tup = read_tuple()
                _ANCHOR_TUPLE = tup
                self.process(tup)
                self.ack(tup)
        except Exception as e:
            self.raise_exception(e, tup)



"""Utilities for interprocess communication between Python and Storm."""
from __future__ import print_function
try:
    import simplejson as json
except ImportError:
    import json
import logging
import os
import sys
from collections import deque


config = context = None
storm_log = logging.getLogger('pystorm')

_MAX_MESSAGE_SIZE = 16777216
_MAX_BLANK_MSGS = 500
_MAX_LINES = 100
# queue up commands we read while trying to read task IDs
_pending_commands = deque()
# queue up task IDs we read while trying to read commands/tuples
_pending_task_ids = deque()
# we'll redirect stdout, but we'll save original for communication to Storm
_stdout = sys.stdout


class StormIPCException(Exception):
    pass


class LogStream(object):
    """Object that implements enough of the Python stream API to be used as
    sys.stdout and sys.stderr. Messages are written to the Python logger.
    """
    def __init__(self, logger):
        self.logger = logger

    def write(self, message):
        for line in message.split('\n'):
            self.logger.error(line)


class Tuple(object):
    """Storm's primitive data type passed around via streams."""

    __slots__ = ['id', 'component', 'stream', 'task', 'values']

    def __init__(self, id, component, stream, task, values):
        self.id = id
        self.component = component
        self.stream = stream
        self.task = task
        self.values = values

    def __repr__(self):
        return '<{}: {}> {}'.format(self.__class__.__name__, self.id,
                                    self.values)


# Message recieving

def read_message_lines():
    lines = blank_lines = 0

    while True:
        line = sys.stdin.readline()[0:-1]  # ignore new line
        if line == 'end':
            break
        lines += 1
        message_size = len(line)

        # If message size exceeds MAX_MESSAGE_SIZE, we assume that the
        # Storm worker has died, and we would be reading an infinite
        # series of blank lines. Throw an error to halt processing,
        # otherwise the task will use 100% CPU and will quickly consume
        # a huge amount of RAM.
        if message_size >= _MAX_MESSAGE_SIZE:
            raise StormIPCException(('Message {} exceeds '
                                     '{:,}'.format(line, _MAX_MESSAGE_SIZE)))

        # If Storm starts to send us a series of blank lines, after awhile we
        # have to assume that the pipe to the Storm supervisor is broken, this
        # is one of the more annoying parts of Petrel and we should probably
        # tune _MAX_BLANK_MSGS
        if line == '':
            blank_lines += 1
            if blank_lines >= _MAX_BLANK_MSGS:
                raise StormIPCException(('{:,} blank lines received, assuming '
                                         'pipe to Storm supervisor is '
                                         'broken'.format(blank_lines)))

        if lines >= _MAX_LINES:
            raise StormIPCException(('Message exceeds {:,} lines, assuming '
                                     'this is an error.'.format(lines)))

        yield line


def read_message():
    """Read a message from Storm, reconstruct newlines appropriately."""
    message = ''.join('{}\n'.format(line) for line in read_message_lines())
    return json.loads(message)


def read_task_ids():
    if _pending_task_ids:
        return _pending_task_ids.popleft()
    else:
        msg = read_message()
        while not isinstance(msg, list):
            _pending_commands.append(msg)
            msg = read_message()
        return msg


def read_command():
    if _pending_commands:
        return _pending_commands.popleft()
    else:
        msg = read_message()
        while isinstance(msg, list):
            _pending_task_ids.append(msg)
            msg = read_message()
        return msg


def read_tuple():
    cmd = read_command()
    return Tuple(cmd['id'], cmd['comp'], cmd['stream'], cmd['task'],
                 cmd['tuple'])


def read_handshake():
    """Read and process an initial handshake message from Storm."""
    # Redirect stdout and stderr to ensure that print statements/functions
    # won't crash the Storm Java worker
    sys.stdout = LogStream(logging.getLogger('pystorm.stdout'))
    sys.stderr = LogStream(logging.getLogger('pystorm.stderr'))

    msg = read_message()
    storm_log.info('Received initial handshake from Storm: %r', msg)
    # Write a blank PID file out to the pidDir
    pid_dir = msg['pidDir']
    pid = os.getpid()
    open('{}/{}'.format(pid_dir, str(pid)), 'w').close()
    send_message({'pid': pid})
    storm_log.info('Process ID sent to Storm')

    return msg['conf'], msg['context']


# Message sending

def send_message(message):
    """Send a message to Storm via stdout"""
    print(json.dumps(message), file=_stdout)
    print('end', file=_stdout)
    _stdout.flush()



"""Base primititve classes for working with Storm."""
import traceback

from ipc import send_message


class Component(object):
    """Base class for Spouts and Bolts which contains class methods for
    logging messages back to the Storm worker process."""

    def raise_exception(self, exception, tup=None):
        """Report an exception back to Storm via logging.

        :param exception: a Python exception.
        :param tup: a :class:`Tuple` object.
        """
        if tup:
            message = ('Python {exception_name} raised while processing tuple '
                       '{tup!r}\n{traceback}')
        else:
            message = 'Python {exception_name} raised\n{traceback}'
        message = message.format(exception_name=exception.__class__.__name__,
                                 tup=tup,
                                 traceback=traceback.format_exc(exception))
        self.log(message)
        send_message({'command': 'sync'})  # sync up right away

    def log(self, message, level='info'):
        """Log a message to Storm optionally providing a logging level.

        :param message: any object that supports ``__str__()``.
        :param level: a ``str`` representing the log level.
        """
        send_message({'command': 'log', 'msg': str(message), 'level': level})



import itertools

from streamparse.spout import Spout

class WordSpout(Spout):

    def initialize(self, stormconf, context):
        self.words = itertools.cycle(['dog', 'cat',
                                      'zebra', 'elephant'])

    def next_tuple(self):
        word = next(self.words)
        self.emit([word])


if __name__ == '__main__':
    WordSpout().run()



from docopt import docopt

from ext.fabric import *
from ext.invoke import *
from bootstrap import quickstart


# XXX: these are commands we're working on still
TODO_CMDS = """
        sparse debug [-e <env>]
        sparse restart [-e <env>]
        sparse attach [-e <env>]
        sparse logs [-e <env>]
"""


def main():
    """sparse: manage streamparse clusters.

    sparse provides a front-end to streamparse, a framework for creating Python
    projects for running, debugging, and submitting computation topologies
    against real-time streams, using Apache Storm.

    It requires the java and lein (Clojure build tool) to be on your $PATH, and
    uses lein and Clojure under the hood for JVM/Thrift interop.

    Usage:
        sparse quickstart <project_name>
        sparse run [-n <topology>] [-t <time>] [-dv]
        sparse submit [-e <env>] [-n <topology>] [-dv]
        sparse list [-e <env>] [-v]
        sparse kill [-n <topology>] [-e <env>] [-v]
        sparse tail [-e <env>]
        sparse (-h | --help)
        sparse --version

    Arguments:
        project_name                The name of your new streamparse project.

    Options:
        -h --help                   Show this screen.
        --version                   Show version.
        -v --verbose                Show verbose output for command.
        -e --environment <env>      The environment to use for the command
                                    corresponding to an environment in your
                                    "envs" dictionary in config.json. If you
                                    only have one environment specified,
                                    streamparse will automatically use this.
        -n --name <topology>        The name of the topology to deploy.  If you
                                    have only one topology defined in your
                                    topologies/ directory, streamparse
                                    will use it automatically.
        -t --time <time>            Time (in seconds) to keep local cluster
                                    running [default: 5].
        -d --debug                  Debug the given command.
    """
    args = docopt(main.__doc__, version="sparse 0.1")

    if args["run"]:
        time = int(args["--time"])
        run_local_topology(args["--name"], time, args["--debug"])
    elif args["list"]:
        list_topologies(args["--environment"])
    elif args["kill"]:
        kill_topology(args["--name"], args["--environment"])
    elif args["quickstart"]:
        quickstart(args['<project_name>'])
    elif args["submit"]:
        submit_topology(args["--name"], args["--environment"])
    elif args["tail"]:
        tail_topology(args["--environment"])


if __name__ == "__main__":
    main()



"""streamparse.ext.fabric

fabric tasks that are imported into a project's fabfile.py.

Should be used like this::

    from fabric.api import *
    from streamparse.ext.fabric import *

    # your other tasks
"""
from __future__ import absolute_import, print_function

from fabric.api import *
from fabric.contrib.files import exists

from .util import get_env_config


__all__ = ["activate_env", "create_or_update_virtualenvs", "tail_logs"]


@task
def _tail_logs(log_path):
    # list log files found
    run("ls {log_path}/*.log".format(log_path=log_path))
    # tail -f all of them
    run("tail -f {log_path}/*.log".format(log_path=log_path))

@task
def tail_logs():
    """Follow (tail -f) the log files on remote Storm workers.

    Will use the `log_path` and `workers` properties from config.json.
    """
    execute(_tail_logs, env.log_path, hosts=env.storm_workers)

@task
def activate_env(env_name=None):
    """Activate a particular environment from a streamparse project's
    config.json file and populatefabric's env dictionary with appropriate
    values.

    :param env_name: a `str` corresponding to the key within the config file's
    "envs" dictionary.
    """
    env_name, env_config = get_env_config(env_name)

    # get the host only (not port) for Nimbus since we'll be using SSH anyway
    # env.storm_nimbus = [env_config["nimbus"].split(":")[0]]
    env.storm_workers = env_config["workers"]
    env.user = env_config["user"]
    env.log_path = env_config["log_path"]
    env.virtualenv_path = env_config["virtualenv_path"]
    env.disable_known_hosts = True


@parallel
def _create_or_update_virtualenv(virtualenv_path, virtualenv_name,
                                requirements_file):
    if not exists("{}/{}".format(virtualenv_path, virtualenv_name)):
        puts("virtualenv not found for {}, creating one.".format(virtualenv_path))
        run("virtualenv {}/{}".format(virtualenv_path, virtualenv_name))

    puts("Uploading requirements.txt to temporary file.")
    tmpfile = run("mktemp /tmp/streamparse_requirements-XXXXXXXXX.txt")
    put(requirements_file, tmpfile)

    puts("Updating virtualenv: {}".format(virtualenv_name))
    cmd = "source {}/{}/bin/activate".format(virtualenv_path, virtualenv_name)
    with prefix(cmd):
        run("pip install streamparse")
        run("pip install -r {}".format(tmpfile))

    run("rm {}".format(tmpfile))


@task
def create_or_update_virtualenvs(virtualenv_name, requirements_file):
    """Create or update virtualenvs on remote servers.

    Assumes that virtualenv is on the path of the remote server(s).

    :param virtualenv_name: the name of the virtualenv.
    :param requirements_file: path to the requirements.txt file to use
    to update/install this virtualenv.
    """
    execute(_create_or_update_virtualenv,
            env.virtualenv_path, virtualenv_name, requirements_file,
            hosts=env.storm_workers)



from __future__ import absolute_import
import json
import os
import sys
from glob import glob

from fabric.colors import red

from ..decorators import memoized


def die(msg, error_code=1):
    print("{}: {}".format(red("error"), msg))
    sys.exit(error_code)


@memoized
def get_config():
    if not os.path.exists("config.json"):
        die("No config.json found. You must run this command inside a "
            "streamparse project directory.")

    with open("config.json") as fp:
        config = json.load(fp)
    return config


def get_topology_definition(topology_name=None):
    """Fetch a topology name and definition file.  If the topology_name is
    None, and there's only one topology definiton listed, we'll select that
    one, otherwise we'll die to avoid ambiguity.

    :param topology_name: a `str`, the topology_name of the topology (without .clj extension).
    :returns: a `tuple` containing (topology_topology_name, topology_file).
    """
    config = get_config()
    topology_path = config["topology_specs"]
    if topology_name is None:
        topology_files = glob("{}/*.clj".format(topology_path))
        if not topology_files:
            die("No topology definitions are defined in {}."
                .format(topology_path))
        if len(topology_files) > 1:
            die("Found more than one topology definition file in {specs_dir}. "
                "When more than one topology definition file exists, you must "
                "explicitly specify the topology by name using the -n or "
                "--name flags.".format(specs_dir=topology_path))
        topology_file = topology_files[0]
        topology_name = topology_file.rstrip(".clj").lstrip(topology_path)
    else:
        topology_file = "{}.clj".format(os.path.join(topology_path, topology_name))
        if not os.path.exists(topology_file):
            die("Topology definition file not found {}. You need to "
                "create a topology definition file first."
                .format(topology_file))

    return (topology_name, topology_file)


def get_env_config(env_name=None):
    """Fetch an environment name and config object from the config.json file.
    If the name is None and there's only one environment, we'll select the
    first, otherwise we'll die to avoid ambiguity.

    :returns: a `tuple` containing (env_name, env_config).
    """
    config = get_config()
    if env_name is None and len(config["envs"]) == 1:
        env_name = config["envs"].keys()[0]
    elif env_name is None and len(config["envs"]) > 1:
        die("Found more than one environment in config.json. "
            "When more than one environment exists, you must "
            "explicitly specify the environment name via the "
            "-e or --environment flags.")
    if env_name not in config["envs"]:
        die("Could not find a \"{}\" in config.json, have you specified one?"
            .format(env_name))

    return (env_name, config["envs"][env_name])


def get_nimbus_for_env_config(env_config):
    """Get the Nimbus server's hostname and port from a streamparse project's
    config file.
    """
    if not env_config["nimbus"]:
        die("No Nimbus server configured for in config.json.")

    if ":" in env_config["nimbus"]:
        host, port = env_config["nimbus"].split(":", 1)
        port = int(port)
    else:
        host = env_config["nimbus"]
        port = 6627

    return (host, port)



from collections import Counter

from streamparse.bolt import Bolt

class WordCounter(Bolt):

    def initialize(self, conf, ctx):
        self.counts = Counter()

    def process(self, tup):
        word = tup.values[0]
        self.counts[word] += 1
        self.emit([word, self.counts[word]])
        self.log('%s: %d' % (word, self.counts[word]))


if __name__ == '__main__':
    WordCounter().run()
